(* ::Package:: *)

BeginPackage["eulerbeamlib`"]


Clamp::usage="Clamp support";
Simple::usage="Simple support";
Free::usage="Free edge";
EulerBeam::usage="EulerBeam[leftBoundary,rightBoundary,length,loadfunction,EIfunction,xvar,assumptions] returns an EulerBeam object";
EulerBeamBoundaryCondition::usage="EulerBeamBoundaryCondition[type, x, functionW, dY: 0, dTheta: 0] returns a list with boundary values for solving the differential equation of beam";
EulerBeamSolve::usage="EulerBeam[beam] solves the beam, returns the solved beam";
EulerBeamReactions::usage="EulerBeamReactions[beam] shows in a table w,slope,moment,shear on the edges of a solved beam";
EulerBeamDiagram::usage="EulerBeamDiagram[beam,type], type can be \"Moment\",\"ShearForce\",\"Slope\",\"Deflection\"";
EulerBeamDeflection::usage="EulerBeamDeflection[beam] returns the deflection function of the solved beam";
EulerBeamSlope::usage="EulerBeamSlope[beam] returns the slope function of the solved beam";
EulerBeamMoment::usage="EulerBeamMoment[beam] returns the moment function of the solved beam";
EulerBeamShearForce::usage="EulerBeamShearForce[beam] returns the shearforce funtion of the solved beam";
EulerBeamValuesComply::usage="EulerBeamRulesComply[values,assumptions], adjust values so that comply with assumptions";
EulerBeamGetDiscontinuity::usage="EulerBeamGetDiscontinuity[expr,xvar,L], returns the expr in parts of the space {0,L} eliminating discontinuities";
EulerBeamGetLimits::usage="EulerBeamGetLimits[beam,type] returns the maxima/minima/inflection points that can find for the beam and the function type";


Begin["`Private`"]


(* calculations *)


EulerBeamBoundaryCondition[type_, x_, wfunction_, dY_: 0, dTheta_: 0] := Module[{p},
   Switch[type,
    	Clamp, p = {wfunction[x] == dY, wfunction'[x] == dTheta},
    	Simple, p = {wfunction[x] == dY, wfunction''[x] == 0},
    	Free, p = {wfunction''[x] == 0, wfunction'''[x] == 0}
    ];
p
];

EulerBeam[leftBoundary_,rightBoundary_,length_,loadfunction_,EIfunction_,xvar_,assumptions_:{}]:=
{{"EulerBeam",leftBoundary,rightBoundary,length,loadfunction,EIfunction,xvar,assumptions},{0,0,0,0}};

EulerBeamSolve::errSolve = "Error in differential equation solution.";
EulerBeamSolve::errBeam = "Error, not an Euler Beam object";
EulerBeamSolve[beam_] := 
  Module[{b,ds, ww, slope,moment,shear,f1,leftB,rightB,length,load,EI,xvar,ass},
   If[beam[[1,1]]!="EulerBeam",Message[EulerBeamDeflection::errBeam]];
   leftB=beam[[1,2]];rightB=beam[[1,3]];length=beam[[1,4]];load=beam[[1,5]];
   EI=beam[[1,6]];xvar=beam[[1,7]];ass=beam[[1,8]];
   (* now solve for deflection ww*)
   f1[xx_] := (EI /. xvar -> xx) * w''[xx];
   ds = {f1''[xvar] == (load /. xvar -> xvar)};
   AppendTo[ds, EulerBeamBoundaryCondition[leftB, 0, w, 0, 0]];
   AppendTo[ds, EulerBeamBoundaryCondition[rightB, length, w, 0, 0]];
   ds = Flatten[ds];
   ww = w[xvar] /. DSolve[ds, w, xvar];
   If[Length[ww] != 1, Message[EulerBeamDeflection::errSolve]];
   ww = ww[[1]]; ww = Simplify[ww, Assumptions -> ass];
   (* compute slope *)
   slope=Simplify[D[ww,xvar],Assumptions->ass];
   (* compute moment *)
   moment=Simplify[- EI D[slope,xvar],Assumptions->ass];
   (* compute shear force *)
   shear=Simplify[D[moment,xvar],Assumptions->ass];

   b=beam; 
   b[[2]]={ww,slope,moment,shear};
   b[[2,1]]=ww;b[[2,2]]=slope;
b
];


EulerBeamReactions[beam_]:=Module[{xvar,length,ass,result,t,l,r},
length=beam[[1,4]];xvar=beam[[1,7]];ass=beam[[1,8]];
result=beam[[2]];
t={};
l=Simplify[(result[[1]]/.xvar->0),Assumptions->ass];
r=Simplify[(result[[1]]/.xvar->length),Assumptions->ass];
AppendTo[t,{"Deflection",l,r}];
l=Simplify[(result[[2]]/.xvar->0),Assumptions->ass];
r=Simplify[(result[[2]]/.xvar->length),Assumptions->ass];
AppendTo[t,{"Slope",l,r}];
l=Simplify[(result[[3]]/.xvar->0),Assumptions->ass];
r=Simplify[(result[[3]]/.xvar->length),Assumptions->ass];
AppendTo[t,{"Moment",l,r}];
l=Simplify[(result[[4]]/.xvar->0),Assumptions->ass];
r=Simplify[(result[[4]]/.xvar->length),Assumptions->ass];
AppendTo[t,{"ShearForce",l,r}];
t 
];


EulerBeamDeflection[beam_]:=beam[[2,1]];
EulerBeamSlope[beam_]:=beam[[2,2]];
EulerBeamMoment[beam_]:=beam[[2,3]];
EulerBeamShearForce[beam_]:=beam[[2,4]];


(*used in Sort to sort x values over the assumption made *)
EulerBeamSortQ[{q1_,q2_},Assumpt_]:=Module[{b},
b=Simplify[q1<q2, Assumptions->Assumpt];
b
];


EulerBeamGetDiscontinuity[expr_,beam_]:=Module[{p,x,f,x1,x2,k,xvar,L,ass,ass2},
xvar=beam[[1,7]];L=beam[[1,4]];ass=beam[[1,8]];
x=Sort[-Union[((Cases[Level[expr,Infinity,Heads->True],_DiracDelta|_HeavisideTheta|_UnitStep])[[All,1]])/.xvar->0]
	,EulerBeamSortQ[{#1,#2},ass]&];
PrependTo[x,0];AppendTo[x,L];
p={};
For[k=1,k<=Length[x]-1,k++,
	x1=x[[k]];x2=x[[k+1]];
	ass2=Flatten[Union[{ass,xvar>x1,xvar<x2}]];
	f=Simplify[expr,Assumptions->ass2];
	AppendTo[p,{{x1,x2},f}];
];
p
];


(* returns a list of {x,f} points of maxima minima *)
EulerBeamGetLimits[beam_,type_]:=Module[{xvar,len,ty,typ,f,df,ff,as1,sol,k,j,xx,p,pp,u},
ty={"Deflection","Slope","Moment","ShearForce"};
typ=Position[ty,type][[1,1]];
f=beam[[2,typ]];
xvar=beam[[1,7]];len=beam[[1,4]];as1=beam[[1,8]];
ff=EulerBeamGetDiscontinuity[f,beam];
p={};
For[j=1,j<=Length[ff],j++,
	f=ff[[j,2]];
	df[x_]:=D[f/.xvar->x,x];
	sol=Quiet[Check[ Solve[D[f,xvar]==0,xvar],
				   Check[ {{xvar->x/.(FindRoot[(D[f/.xvar->x,x])==0,{x,len/2}])}},{{}} ]
				 ]];
	If[Length[sol[[1]] ]!=0,
		For[k=1,k<=Length[sol],k++,
			xx=xvar/.sol[[k]];
			(*xx=Simplify[xx,Assumptions->as1,TimeConstraint->1];*)
			AppendTo[p,xx ];
		];
	];
];
u=Map[#->1&,Union[Select[ Level[f, {-1}], NumericQ[#] == False &]]];
If[Length[Position[u,xvar]]!=0,u=Delete[u,(Position[u,xvar][[1,1]])]];
u=EulerBeamValuesComply[u,as1];
pp={};
For[j=1,j<=Length[p],j++,
	xx=Quiet[Check[N[p[[j]]/.u],0]];
	If[xx!=0 && xx!=(len/.u) && (Element[xx,Reals]),AppendTo[pp,p[[j]] ]];
];
pp
];


(*returns modified values that comply to assumptions *)
EulerBeamValuesComply[values_,assumptions_]:=Module[{k,j,b,u,v,r,xl,xr,xop,b1,a1,f,pos,pos2},
	f[u1_,v1_]:=u1->v1;
	u=Union[Level[values,{-1}][[1;;Length[values]*2;;2]]];
	v=u/.values;
	Label[jump1];
	b=True;
	a1={};
	For[k=1,k<=Length[assumptions],k++,
		a1={};For[j=1,j<=Length[u],j++, AppendTo[a1,u[[j]]->v[[j]]]];
		r=assumptions[[k]];
		xl=r[[1]];xr=r[[2]];xop=r[[0]];	
		pos=Position[u,xl][[1,1]];
		If[(r/.a1)==False,
			b=False;
			Switch[xop,
				Less,
					v[[pos]]=v[[pos]]-(xr/.a1)/2;
				,Greater,
					v[[pos]]=v[[pos]]+(xr/.a1)/2;
				];
		];
	];
	If[b==False,Goto[jump1]];
a1	
];


gClamp[l_,flip_:False]:=Module[{a},{
	If[flip==False, {Line[{{0,-l/2},{0,l/2}}],Table[Rotate[Line[{{-l/2,a},{0,a}}],Pi/6,{0,a}],{a,-l/2,l/2,l/8}]},
		{Line[{{0,-l/2},{0,l/2}}],Table[Rotate[Line[{{l/2,a},{0,a}}],-Pi/6,{0,a}],{a,-l/2,l/2,l/8}]}]
}];
gSimple[l_]:={
	Circle[{0,0},l/8],
	Line[{{0,0},{l/2,-l/2},{-l/2,-l/2},{0,0}}],
	Table[Rotate[Line[{{a,-l/2},{a,-l/2-l/8}}],Pi/6,{a,-l/2}],{a,-l/2,l/2,l/8}]

	
};


Options[EulerBeamDiagram]={
Style->{Dashing[{}],AbsoluteThickness[2.0],Lighter[Black]},
TextStyle->{Black,Larger},TitleStyle->{Black,Large},
BeamStyle->{Dashing[{}],Black,AbsoluteThickness[3.5]},
Precision->1/100,Assumptions->{},Fill->True,Title->"",Values->False,Boundaries->True};

EulerBeamDiagram[beam_,type_,OptionsPattern[]]:=Module[{typ,ty,xvar,g,f,u,ass,fn,t,xx,len,maxt,mint,sizet,r1,dir,
optStyle,optBeamStyle,prec,x0,xlen,p,k,sol,optVal,optBound,x1,x2,leftB,rightB,bb},
optStyle=OptionValue[Style];optBeamStyle=OptionValue[BeamStyle];
optVal=OptionValue[Values];optBound=OptionValue[Boundaries];
ty={"Deflection","Slope","Moment","ShearForce"};
dir={-1,1,-1,1};
ass=OptionValue[Assumptions];
xvar=beam[[1,7]];prec=OptionValue[Precision];
typ=Position[ty,type][[1,1]];
leftB=beam[[1,2]];rightB=beam[[1,3]];
f=beam[[2,typ ]];
u=Map[#->1&,Union[Select[ Level[f, {-1}], NumericQ[#] == False &]]];
If[Length[Position[u,xvar]]!=0,u=Delete[u,(Position[u,xvar][[1,1]])]];
ass=Flatten[AppendTo[ass,u]];
ass=EulerBeamValuesComply[ass,beam[[1,8]]];
fn=(f)/.ass; (* fn holds the f (x) numeric value *)
len=beam[[1,4]]/.ass;
sol=EulerBeamGetDiscontinuity[f,beam];
t={};
For[k=1,k<=Length[sol],k++,
	x1=sol[[k,1,1]];x2=sol[[k,1,2]];
	x0=sol[[k,2]]/.ass;
	t=Join[t,Table[{xx,x0/.xvar->xx},{xx,x1/.ass,x2/.ass,prec}]];
];
PrependTo[t,{0,0}];AppendTo[t,{len,0}];
maxt=Max[t[[All,2]]];mint=Min[t[[All,2]]];sizet=Max[Abs[maxt],Abs[mint],Abs[maxt-mint]];
r1=ScalingTransform[{1,dir[[typ]]*1/sizet*(len/GoldenRatio)}];t=r1[t];
x0=Simplify[f/.xvar->0,Assumptions->beam[[1,8]]];xlen=Simplify[f/.xvar->beam[[1,4]],Assumptions->beam[[1,8]]];
p={Line[{}]};
sol=EulerBeamGetLimits[beam,ty[[typ]]];
For[k=1,k<=Length[sol],k++,
	xx=sol[[k]];
	If[optVal==True,AppendTo[p,Style[Text[xx,{xx/.ass,dir[[typ]]((fn/.xvar->xx)/.ass)/sizet*(len/GoldenRatio)}],OptionValue[TextStyle]]]];
	AppendTo[p,Dashed];AppendTo[p,Thin];
	AppendTo[p,Line[{{xx/.ass,0},{xx/.ass,dir[[typ]]((fn/.xvar->xx)/.ass)/sizet*(len/GoldenRatio)}}]];
];
bb={Line[{}]};
If[optBound==True,
	If[leftB==Clamp,AppendTo[bb,gClamp[len/10,False]]];
	If[rightB==Clamp,AppendTo[bb,Translate[gClamp[len/10,True],{len,0}]]];
	If[leftB==Simple,AppendTo[bb,gSimple[len/10]]];
	If[rightB==Simple,AppendTo[bb,Translate[gSimple[len/10],{len,0}]]];
];
g=Graphics[{Thick,bb,
Flatten[{optBeamStyle,Line[{{0,0},{len,0}}]}],
Flatten[{optStyle,Line[t]}],If[OptionValue[Fill]==True,Flatten[{optStyle,Opacity[0.3],Polygon[t],Opacity[1]}],Line[{}]],
Style[Text[If[OptionValue[Title]!="",OptionValue[Title],ty[[typ]]],{len/2,len/GoldenRatio}],OptionValue[TitleStyle]],
If[optVal==True,{
Style[Text[x0,{0,dir[[typ]](fn/.xvar->0)/sizet*(len/GoldenRatio)}],OptionValue[TextStyle] ],
Style[Text[xlen,{len,dir[[typ]](fn/.xvar->len)/sizet*(len/GoldenRatio)}],OptionValue[TextStyle] ]},Line[{}] ],
p
},{PlotRange->All}];
g
]





End[]


EndPackage[]
